﻿using AdventOfCode.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;

namespace AdventOfCode._2023
{
    public class Day2 : AOCDay
    {
        private readonly List<Cube> ANSWER_CUBES = new List<Cube>()
        {
            new Cube("12 red"),
            new Cube("13 green"),
            new Cube("14 blue")
        };

        protected override AOCResponse ExecutePartA()
        {
            //Create game objects for each of our lines
            var games = this.GetSplitInput().Select(x => new Game(x)).ToList();

            //Select only our valid games
            var validGames = games.Where(x => x.CompareAgainstAnswer(ANSWER_CUBES));

            this.Answer = validGames.Sum(x => x.Id);
            return this._response;
        }

        protected override AOCResponse ExecutePartB()
        {
            //Create game objects for each of our lines
            var games = this.GetSplitInput().Select(x => new Game(x)).ToList();

            if (!this._request.IgnoreLogMessages)
            {
                games.ForEach(x =>
                {
                    var minAnswerCubes = x.GetMinAnswerCubes();
                    Log($"Game ID {x.Id} generated the min answer: {string.Join(", ", minAnswerCubes.Select(x => $"{x.Count} {x.Color}"))} with power {x.GetGamePower()}");
                });
            }
            this.Answer = games.Sum(x => x.GetGamePower());
            return this._response;
        }
    }

    class Game
    {
        public int Id { get; set; }
        public List<Drawing>  drawings { get; set; } = new List<Drawing>();
        public Game(string input)
        {
            var splitInput = input.Split(':');
            Id = int.Parse(splitInput[0].Split(' ')[1]); //Extract our game id out of input string

            //Parse our drawings into objects
            foreach (var drawing in splitInput[1].Split(';'))
            {
                drawings.Add(new Drawing(drawing.Trim()));
            }
        }

        public bool CompareAgainstAnswer(List<Cube> baggedCubes)
        {
            foreach (var drawing in drawings)
            {
                foreach (var cube in drawing.Cubes)
                {
                    var answerCube = baggedCubes.First(x => x.Color == cube.Color);
                    if (cube.Count > answerCube.Count)
                    {
                        //Means the drawed cube count exceeds the number of cubes in our bag
                        return false;
                    }
                }
            }

            return true;
        }

        public List<Cube> GetMinAnswerCubes()
        {
            var minAnswerCubes = new List<Cube>();
            foreach (var drawing in drawings)
            {
                foreach (var cube in drawing.Cubes)
                {
                    var matchedCube = minAnswerCubes.FirstOrDefault(x => x.Color == cube.Color);
                    if (matchedCube == null)
                    {
                        minAnswerCubes.Add(new Cube()
                        {
                            Color = cube.Color,
                            Count = cube.Count
                        });
                    }
                    else
                    {
                        matchedCube.Count = Math.Max(matchedCube.Count, cube.Count);
                    }
                }
            }
            return minAnswerCubes;
        }

        public int GetGamePower()
        {
            var minAnswerCubes = GetMinAnswerCubes();
            var power = 1;
            minAnswerCubes.ForEach(x => power *= x.Count);
            return power;
        }
    }

    class Drawing
    {
        public List<Cube> Cubes { get; set; } = new List<Cube>();

        public Drawing(string drawing)
        {
            foreach (var input in drawing.Split(','))
            {
                Cubes.Add(new Cube(input.Trim()));
            }
        }
    }

    class Cube
    {
        public int Count { get; set; }
        public string Color { get; set; }

        public Cube() { }
        public Cube(string input)
        {
            var split = input.Split(' ');
            Count = int.Parse(split[0]);
            Color = split[1];
        }
    }
}
